from rest_framework import serializers
from wagtail.blocks import RichTextBlock

from wagtail.templatetags.wagtailcore_tags import richtext as richtext_filter
from django.utils.translation import gettext as _



def rich_text(add_features: [str]):
    return RichTextSerializer.block_definition(add_features)


class RichTextSerializer(serializers.Serializer):
    block_name = 'richtext'
    text = serializers.SerializerMethodField('get_text')

    @staticmethod
    def block_definition(add_features: [str] = []):
        features = ['h2', 'h3', 'italic', 'bold', 'ol', 'ul', 'hr', 'link',
                    'document-link', 'image', 'embed', 'code',
                    'superscript', 'subscript', 'strikethrough', 'blockquote']
        features.extend(add_features)
        return RichTextSerializer.block_name, RichTextBlock(features=features,icon='doc-full')

    class Meta:
        fields = ('text',)

    def get_text(self, value):
        return richtext_filter(value.source)



from django.conf import settings
from rest_framework import serializers
from wagtail import blocks
from django.utils.translation import gettext_lazy as _
from wagtail.blocks import StreamBlock

from wagtail_rest_pack.streamfield.serializers import SettingsStreamFieldSerializer

rest_pack = getattr(settings, "REST_PACK", {})
icons = rest_pack.get('icons', [])


class LinkCategorySerializer(serializers.Serializer):
    block_name = 'linkcategory'

    name = serializers.CharField(max_length=50)
    icon = serializers.CharField()
    stream = SettingsStreamFieldSerializer()

    @staticmethod
    def block_definition(local_blocks):
        return LinkCategorySerializer.block_name, blocks.StructBlock(local_blocks=[
            ('name', blocks.TextBlock(max_length=50, required=True, help_text=_('Name of the category'))),
            ('stream', StreamBlock(local_blocks, label=_('A nested content of a category'))),
            ('icon', blocks.ChoiceBlock(choices=[('none', _('None'))] + icons, default=['none'], label=_('The icon'))),
        ])

    class Meta:
        fields = ['name', 'stream', 'icon']


class LinkBlockSerializer(serializers.Serializer):
    block_name = 'linkblock'

    name = serializers.CharField(max_length=50)
    icon = serializers.CharField()
    page = serializers.SerializerMethodField('get_page_repre')

    @staticmethod
    def block_definition(nested:bool=False):
        local_block_name = LinkBlockSerializer.block_name
        if nested:
            local_block_name = 'nested'+local_block_name
        return local_block_name, blocks.StructBlock(local_blocks=[
            ('name', blocks.TextBlock(max_length=50, required=True, help_text=_('Name of the link'))),
            ('page', blocks.PageChooserBlock(label=_('A page to be opened'))),
            ('icon', blocks.ChoiceBlock(choices=[('none', _('None'))] + icons, default=['none'], label=_('The icon'))),
        ])

    def get_page_repre(self, value):
        page = value['page']
        return {
            'id': page.id,
            'url': page.url,
        }

    class Meta:
        fields = ['name', 'page', 'icon']



class ChildrenLinksSerializer(serializers.Serializer):
    block_name = 'categorychildren'
    children = serializers.SerializerMethodField('get_page_repre')

    @staticmethod
    def block_definition():
        return ChildrenLinksSerializer.block_name, blocks.StructBlock(local_blocks=[
            ('parent', blocks.PageChooserBlock(label=_('A page which children should be listed'))),
        ])

    def get_page_repre(self, value):
        parent = value['parent']
        children_qs = parent.specific.children
        # todo limit somehow, or filter according to "show_in_menus"
        children = map(lambda child: {'name': child.title, 'page': child, 'icon':''}, children_qs)
        return LinkBlockSerializer(instance=children, many=True).data

    class Meta:
          fields = ['children',]


# Generated by Django 4.1.2 on 2022-10-17 17:13

from django.db import migrations
import wagtail.blocks
import wagtail.fields
import wagtail.images.blocks


class Migration(migrations.Migration):

    dependencies = [
        ('streamfield', '0010_auto_20210512_1426'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='gallery',
            options={'verbose_name': 'Galerie', 'verbose_name_plural': 'Galerie'},
        ),
        migrations.AlterField(
            model_name='gallery',
            name='stream',
            field=wagtail.fields.StreamField([('gallery_image', wagtail.blocks.StructBlock([('id', wagtail.images.blocks.ImageChooserBlock(icon='image', label='Image'))]))], use_json_field=True),
        ),
    ]




from django.core.exceptions import ValidationError
from wagtail.admin.mail import send_mail
from django.template.loader import render_to_string
from django.utils.translation import gettext as _

class EmailActionHandler:

    def __init__(self, **kwargs):
        self.context = kwargs

    def handle(self, value):
        form = self.context['form']
        form_name = form.name
        subject = _(f'Odpověď na formulář "{form_name}"')
        sender_field_name = value['sender']
        validated_data = self.context['validated_data']
        sender = validated_data.get(sender_field_name, None)
        if sender is None and self.context['request'].user.is_authenticated:
            sender = self.context['request'].user.email
        context= {
            'user': self.context['request'].user,
            'data': validated_data,
            'context': {}
        }
        msg_html = render_to_string('send_email_action.html', context)
        send_mail(subject=subject,
                  message=str(validated_data),
                  from_email=sender,
                  recipient_list=[value['address']],
                  html_message=msg_html)



from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class CustomPageTagConfig(AppConfig):
    name = 'wagtail_rest_pack.custom_tag'
    verbose_name = _('Custom Page Tag')

    def ready(self):
        pass



import re

from wagtail_rest_pack.relations.models import Relation

page_id_pattern = re.compile('id="([0-9]+)" linktype="page"')


def indexUpdated(**kwargs):
    instance = kwargs['instance']
    if not instance.has_unpublished_changes:
        ids = []
        if hasattr(instance, 'relation_fields'):
            # todo use texteditor handlers if possible
            for field in instance.relation_fields:
                value = field.serializer.to_representation(getattr(instance, field.name))
                ids.extend(page_id_pattern.findall(value))
        relations = []
        for id in set(ids):
            relation = Relation()
            relation.from_page = instance
            relation.to_page_id = id
            relations.append(relation)
        Relation.objects.filter(from_page_id=instance.id).delete()
        for relation in relations:
            relation.save()


def indexDeleted(**kwargs):
    instance = kwargs['instance']
    Relation.objects.filter(to_page_id=instance.id).delete()
    Relation.objects.filter(from_page_id=instance.id).delete()



from wagtail import hooks
from .handlers import ThumbnailedEmbedHandler

@hooks.register('register_rich_text_features')
def register_embed_handler(features):
    features.register_embed_type(ThumbnailedEmbedHandler)



# Generated by Django 4.1.2 on 2022-10-17 17:13

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('sitesettings', '0002_auto_20210510_1431'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='imageslideritem',
            options={'verbose_name': 'Slider', 'verbose_name_plural': 'Sliders'},
        ),
        migrations.AlterModelOptions(
            name='sitesettingssection',
            options={'verbose_name': 'Slider', 'verbose_name_plural': 'Sliders'},
        ),
    ]



from django.utils.translation import gettext_lazy as _
from rest_framework import serializers
from wagtail import blocks
from wagtail.blocks import PageChooserBlock


def page_list():
    return PageListSerializer.block_definition()


page_list_variants = [
    ('simple', _('Simple')),
    ('amazing', _('Top, Top Three and simple')),
    ('nested', _('Directly draw all pages inside')),
]


class PageListSerializer(serializers.Serializer):
    block_name = 'pagelist'
    variant = serializers.ChoiceField(page_list_variants)
    children_of = serializers.SerializerMethodField('get_page_id')

    @staticmethod
    def block_definition():
        # _('List children of selected page') todo translate
        return PageListSerializer.block_name, PageListStruct(label="Seznam stránek")

    class Meta:
        fields = ('variant', 'children_of',)

    def get_page_id(self, value):
        return value['children_of'].id

class PageListStruct(blocks.StructBlock):
    def __init__(self, *args, **kwargs):
        super().__init__(local_blocks=[
            ('children_of', PageChooserBlock(label=_('List children of selected page'))),
            ('variant',
             blocks.ChoiceBlock(choices=page_list_variants, label=_('A variant how children should be displayed.')))
        ], icon='folder-open-1', **kwargs)



# Generated by Django 3.1.10 on 2021-05-11 16:41

from django.db import migrations, models
import wagtail.blocks
import wagtail.fields
import wagtail.images.blocks


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('streamfield', '0003_delete_gallery'),
    ]

    operations = [
        migrations.CreateModel(
            name='Gallery',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(help_text='Gallery name', max_length=120)),
                ('stream', wagtail.fields.StreamField([('gallery_image', wagtail.blocks.StructBlock([('image', wagtail.images.blocks.ImageChooserBlock())], icon='image', label='Image'))])),
            ],
            options={
                'verbose_name': 'Gallery',
                'verbose_name_plural': 'Galleries',
            },
        ),
    ]




from rest_framework import serializers
from wagtail import blocks
from django.utils.translation import gettext_lazy as _
from wagtail.blocks import StreamBlock
from wagtail.snippets.blocks import SnippetChooserBlock
from wagtail_rest_pack.streamfield.models import Gallery

from wagtail_rest_pack.streamfield.serializers import SettingsStreamFieldSerializer, get_stream_field_serializers



def gallery_block():
    return GallerySerializer.block_definition()


class GallerySerializer(serializers.ModelSerializer):
    block_name = 'gallery'
    images = serializers.SerializerMethodField('get_images')

    @staticmethod
    def block_definition():
        # _('Gallery') todo translate
        return GallerySerializer.block_name, SnippetChooserBlock(target_model=Gallery, label="Galerie", icon='image')

    class Meta:
        model = Gallery
        fields = ['name', 'images', ]

    def get_images(self, gallery):
        data = gallery.stream.raw_data
        serializers = get_stream_field_serializers()
        for item in data:
            cls = serializers.get(item['type'])
            assert cls is not None, (
                '%s serializers is not registered.' % item['type']
            )
            yield cls(item['value']).data




from rest_framework import generics
from wagtail.models import Page
from wagtail.search.utils import parse_query_string
from django.http import HttpResponseNotFound, HttpResponseBadRequest, HttpResponse
import json

from wagtail_rest_pack.custom_tag.models import PageTag
from wagtail_rest_pack.page_banner.serializers import BanneredChildrenSerializer
from wagtail.images import get_image_model
import time

from wagtail_rest_pack.streamfield.image import GalleryImageSerializer


class SearchView(generics.RetrieveAPIView):
    def get_serializer_class(self):
        return BanneredChildrenSerializer

    def get_pages_queryset(self):
        return Page.objects.live().public()

    def get_image_queryset(self):
        return get_image_model().objects.all()

    def measure(self, t0):
        return round(time.time() - t0, 3)

    def get(self, request, *args, **kwargs):
        query_string = request.GET.get('q')
        if query_string is None:
            return HttpResponseBadRequest()
        filters, query = parse_query_string(query_string)
        type = filters.get('type')
        if type not in ['pages', 'images', 'documents', 'tags']:
            return HttpResponseBadRequest()
        t0 = time.time()
        if type == 'pages':
            pages = self.get_pages_queryset().specific().search(query)
            pages = self.paginate_queryset(pages)
            ser = BanneredChildrenSerializer()
            ser._context = self.get_serializer_context()
            result = {
                'pages': ser.to_representation(pages),
                'time': self.measure(t0)
            }
        if type == 'images':
            images = self.get_image_queryset().values('id').search(query)
            images = self.paginate_queryset(images)
            result = {
                'images': GalleryImageSerializer(many=True).to_representation(images),
                'time': self.measure(t0)
            }
        if type == 'documents':
            documents = []
            result = {
                'documents': documents,
                'time': self.measure(t0)
            }
        if type == 'tags':
            tags = set(query.query_string.split(' '))
            pages = Page.objects.live().public().filter(id__in=PageTag.objects.filter(tag__name__in=tags).values_list('content_object')).specific()
            pages = self.paginate_queryset(pages)
            ser = BanneredChildrenSerializer()
            ser._context = self.get_serializer_context()
            result = {
                'tags': ser.to_representation(pages),
                'time': self.measure(t0)
            }
        return HttpResponse(json.dumps(result), content_type="application/json")


from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class CommentsConfig(AppConfig):
    name = 'wagtail_rest_pack.comments'
    verbose_name = _('Comments')

    def ready(self):
        pass



from django.db import models
from wagtail.api import APIField

from wagtail_rest_pack.breadcrumps.serializers import BreadcrumpSerializer


class BreadcrumpPageMixin(models.Model):

    api_fields= [
        APIField('breadcrumps', BreadcrumpSerializer())
    ]

    @property
    def breadcrumps(self):
        return self

    class Meta:
        abstract= True
